@@ -1,6 +1,6 @@
 /* EtherApe
  * Copyright (C) 2001 Juan Toledo
- * $Id: capture.c,v 1.144 2001/08/08 17:00:22 toledo Exp $
+ * $Id: capture.c,v 1.148.2.3 2003/02/25 00:05:29 bchiara Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -177,7 +177,7 @@ init_capture (void)
 	error = TRUE;
       l3_offset = 4;
       break;
-#ifdef L_LINUX_SLL
+#ifdef DLT_LINUX_SLL
     case L_LINUX_SLL:		/* Linux cooked sockets (I believe this
 				 * is used for ISDN on linux) */
       g_my_info (_("Link type is Linux cooked sockets"));
@@ -203,6 +203,7 @@ init_capture (void)
       else
 	{
 	  g_free (pref.filter);
+	  pref.filter = NULL;
 	  str = g_strdup ("ip");
 	}
       break;
@@ -212,6 +213,7 @@ init_capture (void)
       else
 	{
 	  g_free (pref.filter);
+	  pref.filter = NULL;
 	  str = g_strdup ("tcp");
 	}
       break;
@@ -221,6 +223,7 @@ init_capture (void)
       else
 	{
 	  g_free (pref.filter);
+	  pref.filter = NULL;
 	  str = g_strdup ("udp");
 	}
       break;
@@ -233,7 +236,8 @@ init_capture (void)
 	str = g_strdup (pref.filter);
       break;
     }
-  g_free (pref.filter);
+  if (pref.filter)
+    g_free (pref.filter);
   pref.filter = str;
   str = NULL;
 
@@ -336,6 +340,7 @@ set_filter (gchar * filter_string, gchar * device)
 gboolean
 start_capture (void)
 {
+  GnomeCanvas *gc;
 
   if ((status != PAUSE) && (status != STOP))
     {
@@ -364,6 +369,11 @@ start_capture (void)
 					   (GDestroyNotify) cap_t_o_destroy);
     }
 
+  /* set the antialiasing */
+  gc = GNOME_CANVAS (glade_xml_get_widget (xml, "canvas1"));
+  if (gc)
+    gc->aa = pref.antialias;
+
   status = PLAY;
   return TRUE;
 }				/* start_capture */
@@ -821,10 +831,7 @@ create_node (const guint8 * packet, const guint8 * node_id)
   /* We have already allocated memory for the id when we created the
    * packet. We will use that, and will free it when the node disappears
    * and not with the packet */
-#if 0
-  node->node_id = g_memdup (node_id, node_id_length);
-#endif
-  node->node_id = node_id;
+  node->node_id = (guint8 *) node_id;	/* cast away const */
 
   node->name = NULL;
   node->numeric_name = NULL;
@@ -996,6 +1003,13 @@ update_nodes (void)
 {
   guint n_nodes_before, n_nodes_after;
 
+  /* We have to make sure that all packets have been processed before
+   * updating the nodes itself, because packets rely on the nodes
+   * node_id, and if a node were deleted, subsequent packet list
+   * updating could be corrupt */
+  g_tree_traverse (nodes, (GTraverseFunc) update_node_packets, G_IN_ORDER,
+		   NULL);
+
   do
     {
       n_nodes_before = g_tree_nnodes (nodes);
@@ -1006,6 +1020,18 @@ update_nodes (void)
 
 }				/* update_nodes */
 
+/* Removes old packets from a node */
+static gint
+update_node_packets (guint8 * node_id, node_t * node, gpointer pointer)
+{
+
+  if (node->packets)
+    update_packet_list (node->packets, (gpointer) node, NODE);
+
+  return FALSE;
+}				/* update_node_packets */
+
+
 /* Deletes all data from a node, and possibly the node itself */
 static gint
 update_node (guint8 * node_id, node_t * node, gpointer pointer)
@@ -1015,10 +1041,6 @@ update_node (guint8 * node_id, node_t * node, gpointer pointer)
   protocol_t *protocol_info = NULL;
   guint i = STACK_SIZE;
 
-  if (node->packets)
-    update_packet_list (node->packets, (gpointer) node, NODE);
-
-
   if (node->n_packets == 0)
     {
 
@@ -1104,7 +1126,11 @@ update_node (guint8 * node_id, node_t * node, gpointer pointer)
 	    /* Remove all mentions to this node in the globals protocols list */
 	    forget_node_from_protocols (node_id);
 
+	    /* TODO 
+	     * I believe there might be instances where the node_id is still
+	     * tried to be used, by check_packet, in particular.  */
 	    g_free (node_id);
+
 	    node = NULL;
 	    return TRUE;	/* I've checked it's not safe to traverse 
 				 * while deleting, so we return TRUE to stop
