@@ -1,6 +1,6 @@
 /* EtherApe
  * Copyright (C) 2001 Juan Toledo
- * $Id: diagram.c,v 1.119 2001/08/12 14:34:33 toledo Exp $
+ * $Id: diagram.c,v 1.121.2.3 2003/02/25 00:05:29 bchiara Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -86,14 +86,11 @@ init_diagram ()
   widget = glade_xml_get_widget (xml, "stack_level_menu");
   gtk_option_menu_set_history (GTK_OPTION_MENU (widget), pref.stack_level);
   widget = glade_xml_get_widget (xml, "filter_gnome_entry");
-  gnome_entry_load_history (GNOME_ENTRY (widget));
   widget = glade_xml_get_widget (xml, "file_filter_entry");
-  gnome_entry_load_history (GNOME_ENTRY (widget));
   widget = glade_xml_get_widget (xml, "fileentry");
   widget = gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY (widget));
-  gnome_entry_load_history (GNOME_ENTRY (widget));
 
-  load_color_clist ();		/* Updates the color preferences table with pref.colors */
+  load_color_list ();		/* Updates the color preferences table with pref.colors */
 
   /* Connects signals */
   widget = glade_xml_get_widget (xml, "node_radius_slider");
@@ -148,6 +145,8 @@ init_diagram ()
 
   /* Sets canvas background to black */
   canvas = glade_xml_get_widget (xml, "canvas1");
+//r.g.  gc = GNOME_CANVAS(canvas);  
+
   gdk_color_parse ("black", &black_color);
   gdk_colormap_alloc_color (gdk_colormap_get_system (), &black_color,
 			    TRUE, TRUE);
@@ -374,15 +373,11 @@ check_new_protocol (protocol_t * protocol, GtkWidget * canvas)
   protocol_t *legend_protocol = NULL;
   GtkWidget *prot_table;
   GtkWidget *label;
-  GtkArg args[2];
   GtkStyle *style;
   gchar *color_string;
-  guint n_rows, n_columns;
+  guint n_rows = 1, n_columns = 1;
   static gboolean first = TRUE;
 
-  args[0].name = "n_rows";
-  args[1].name = "n_columns";
-
   /* First, we check whether the diagram already knows about this protocol,
    * checking whether it is shown on the legend. */
   /*  g_message ("Looking for %s", protocol->name); */
@@ -401,9 +396,9 @@ check_new_protocol (protocol_t * protocol, GtkWidget * canvas)
   /* It's not, so we build a new entry on the legend */
   /* First, we add a new row to the table */
   prot_table = glade_xml_get_widget (xml, "prot_table");
-  gtk_object_getv (GTK_OBJECT (prot_table), 2, args);
-  n_rows = args[0].d.int_data;
-  n_columns = args[0].d.int_data;
+  g_object_get (G_OBJECT (prot_table), "n_rows", &n_rows, "n_columns",
+		&n_columns, NULL);
+
   /* Glade won't let me define a 0 row table
    * I feel this is ugly, but it's late and I don't feel like
    * cleaning this up :-) */
@@ -428,7 +423,7 @@ check_new_protocol (protocol_t * protocol, GtkWidget * canvas)
 		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 		    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
   gtk_table_resize (GTK_TABLE (prot_table), n_rows + 1, n_columns - 1);
-  gtk_container_queue_resize (GTK_CONTAINER (app1));
+  gtk_widget_queue_resize (GTK_WIDGET (app1));
 
 
   color_string = get_prot_color (protocol->name);
@@ -505,7 +500,6 @@ get_prot_color (gchar * name)
   static gchar *protocol = NULL;
   gint i = 0;
 
-
   /* Default is to assign the next color in cycle as long
    * as cycling assigned protocols is set.
    * If it's not, then search for a not assigned color. */
@@ -583,14 +577,13 @@ check_new_node (guint8 * node_id, node_t * node, GtkWidget * canvas)
 				 "x2", 0.0,
 				 "y1", 0.0,
 				 "y2", 0.0,
-				 "fill_color",
-				 pref.node_color,
-				 "outline_color",
-				 "black", "width_pixels", 0, NULL);
+				 "fill_color", pref.node_color,
+				 "outline_color", "black",
+				 "width_pixels", 0, NULL);
       gtk_object_ref (GTK_OBJECT (new_canvas_node->node_item));
       new_canvas_node->text_item =
-	gnome_canvas_item_new (group, GNOME_TYPE_CANVAS_TEXT, "text",
-			       node->name->str,
+	gnome_canvas_item_new (group, GNOME_TYPE_CANVAS_TEXT,
+			       "text", node->name->str,
 			       "x", 0.0,
 			       "y", 0.0,
 			       "anchor", GTK_ANCHOR_CENTER,
@@ -634,12 +627,12 @@ update_canvas_nodes (guint8 * node_id, canvas_node_t * canvas_node,
 {
   node_t *node;
   gdouble node_size;
-  GtkArg args[1];
   GList *protocol_item;
   protocol_t *protocol = NULL;
   static clock_t start = 0;
   clock_t end;
   gdouble cpu_time_used;
+  char *nametmp = NULL;
 
   /* We don't need this anymore since now update_nodes is called in update_diagram */
 #if 0
@@ -722,23 +715,35 @@ update_canvas_nodes (guint8 * node_id, canvas_node_t * canvas_node,
 
 
   if (protocol)
-    canvas_node->color = protocol->color;
+    {
+      canvas_node->color = protocol->color;
 
-  gnome_canvas_item_set (canvas_node->node_item,
-			 "x1", -node_size / 2,
-			 "x2", node_size / 2,
-			 "y1", -node_size / 2,
-			 "y2", node_size / 2,
-			 "fill_color_gdk", &(canvas_node->color), NULL);
+      gnome_canvas_item_set (canvas_node->node_item,
+			     "x1", -node_size / 2,
+			     "x2", node_size / 2,
+			     "y1", -node_size / 2,
+			     "y2", node_size / 2,
+			     "fill_color_gdk", &(canvas_node->color), NULL);
+    }
+  else
+    {
+      guint32 black = 0x000000ff;
+      gnome_canvas_item_set (canvas_node->node_item,
+			     "x1", -node_size / 2,
+			     "x2", node_size / 2,
+			     "y1", -node_size / 2,
+			     "y2", node_size / 2,
+			     "fill_color_rgba", black, NULL);
+    }
 
   /* We check the name of the node, and update the canvas node name
    * if it has changed (useful for non blocking dns resolving) */
   /*TODO why is it exactly that sometimes it is NULL? */
   if (canvas_node->text_item)
     {
-      args[0].name = "text";
-      gtk_object_getv (GTK_OBJECT (canvas_node->text_item), 1, args);
-      if (strcmp (args[0].d.string_data, node->name->str))
+      g_object_get (G_OBJECT (canvas_node->text_item), "text", &nametmp,
+		    NULL);
+      if (strcmp (nametmp, node->name->str))
 	{
 	  gnome_canvas_item_set (canvas_node->text_item,
 				 "text", node->name->str, NULL);
@@ -746,7 +751,7 @@ update_canvas_nodes (guint8 * node_id, canvas_node_t * canvas_node,
 	}
 
       /* Memprof is telling us that we have to free the string */
-      g_free (args[0].d.string_data);
+      g_free (nametmp);
     }
 
   /* Processor time check. If too much time has passed, update the GUI */
@@ -1072,27 +1077,25 @@ update_canvas_links (guint8 * link_id, canvas_link_t * canvas_link,
   canvas_node_t *canvas_node;
   GList *protocol_item;
   protocol_t *protocol = NULL;
-  GtkArg args[2];
   gdouble link_size, versorx, versory, modulus;
   struct timeval diff;
   guint32 scaledColor;
   gdouble scale;
+  double dx, dy;		/* temporary */
 
-  link = canvas_link->link;
-
-#if 0
-  /* First we check whether the link has timed out */
-  link = update_link (link);
-#endif
+/* We used to run update_link here, but that was a major performance penalty, and now it is done in update_diagram */
   link = g_tree_lookup (links, link_id);
 
   if (!link)
     {
-      /* TODO VERY IMPORTANT *
-       * Make the same destroying procedure as in the canvas_nodes
-       * Else we are prone to have memory corruption */
-      gtk_object_destroy (GTK_OBJECT (canvas_link->link_item));
-      gtk_object_unref (GTK_OBJECT (canvas_link->link_item));
+      /* Right now I'm not very sure in which cases there could be a canvas_link but not a link_item, but
+       * I had a not in update_canvas_nodes that if the test is not done it can lead to corruption */
+      if (canvas_link->link_item)
+	{
+	  gtk_object_destroy (GTK_OBJECT (canvas_link->link_item));
+	  gtk_object_unref (GTK_OBJECT (canvas_link->link_item));
+	  canvas_link->link_item = NULL;
+	}
 
       g_tree_remove (canvas_links, link_id);
       g_my_debug ("Removing canvas link. Number of links %d",
@@ -1104,12 +1107,9 @@ update_canvas_links (guint8 * link_id, canvas_link_t * canvas_link,
 				 * the traversion (Does that word exist? :-) */
     }
 
+  diff = substract_times (now, link->last_time);
 
-  /* TODO do we really have to check for link in this two? */
-  if (link)
-    diff = substract_times (now, link->last_time);
-
-  if (link && link->main_prot[pref.stack_level])
+  if (link->main_prot[pref.stack_level])
     {
       protocol_item = g_list_find_custom (protocols[pref.stack_level],
 					  link->main_prot[pref.stack_level],
@@ -1119,8 +1119,7 @@ update_canvas_links (guint8 * link_id, canvas_link_t * canvas_link,
       else
 	g_warning (_("Main link protocol not found in update_canvas_links"));
     }
-  args[0].name = "x";
-  args[1].name = "y";
+
 
   points = gnome_canvas_points_new (3);
 
@@ -1137,9 +1136,8 @@ update_canvas_links (guint8 * link_id, canvas_link_t * canvas_link,
       gnome_canvas_points_unref (points);
       return FALSE;
     }
-  gtk_object_getv (GTK_OBJECT (canvas_node->group_item), 2, args);
-  points->coords[0] = args[0].d.double_data;
-  points->coords[1] = args[1].d.double_data;
+  g_object_get (G_OBJECT (canvas_node->group_item), "x", &points->coords[0],
+		"y", &points->coords[1], NULL);
 
   /* We get coords from source node */
   canvas_node = g_tree_lookup (canvas_nodes, link_id);
@@ -1149,48 +1147,62 @@ update_canvas_links (guint8 * link_id, canvas_link_t * canvas_link,
       gnome_canvas_points_unref (points);
       return FALSE;
     }
-  gtk_object_getv (GTK_OBJECT (canvas_node->group_item), 2, args);
 
-  versorx = -(points->coords[1] - args[1].d.double_data);
-  versory = points->coords[0] - args[0].d.double_data;
+  g_object_get (G_OBJECT (canvas_node->group_item), "x", &dx, "y", &dy, NULL);
+  versorx = -(points->coords[1] - dy);
+  versory = points->coords[0] - dx;
+
   modulus = sqrt (pow (versorx, 2) + pow (versory, 2));
   link_size = get_link_size (link->average) / 2;
 
-  points->coords[2] = args[0].d.double_data + (versorx / modulus) * link_size;
-  points->coords[3] = args[1].d.double_data + (versory / modulus) * link_size;
-  points->coords[4] = args[0].d.double_data - (versorx / modulus) * link_size;
-  points->coords[5] = args[1].d.double_data - (versory / modulus) * link_size;
+  points->coords[2] = dx + (versorx / modulus) * link_size;
+  points->coords[3] = dy + (versory / modulus) * link_size;
+  points->coords[4] = dx - (versorx / modulus) * link_size;
+  points->coords[5] = dy - (versory / modulus) * link_size;
 
   /* TODO What if there never is a protocol?
    * I have to initialize canvas_link->color to a known value */
   if (protocol)
-    canvas_link->color = protocol->color;
-
-  if (pref.nofade)
     {
-      guint32 color;
-      color = (((int) (canvas_link->color.red) & 0xFF00) << 16) |
-	(((int) (canvas_link->color.green) & 0xFF00) << 8) |
-	((int) (canvas_link->color.blue) & 0xFF00) | 0xFF;
-      gnome_canvas_item_set (canvas_link->link_item,
-			     "points", points,
-			     "fill_color_rgba", color, NULL);
+      canvas_link->color = protocol->color;
+
+      if (pref.nofade)
+	{
+	  guint32 color;
+	  color = (((int) (canvas_link->color.red) & 0xFF00) << 16) |
+	    (((int) (canvas_link->color.green) & 0xFF00) << 8) |
+	    ((int) (canvas_link->color.blue) & 0xFF00) | 0xFF;
+	  gnome_canvas_item_set (canvas_link->link_item,
+				 "points", points,
+				 "fill_color_rgba", color, NULL);
+	}
+      else
+	{
+	  /* scale color down to 10% at link timeout */
+	  scale =
+	    pow (0.10,
+		 (diff.tv_sec * 1000.0 +
+		  diff.tv_usec / 1000) / pref.link_timeout_time);
+	  scaledColor =
+	    (((int) (scale * canvas_link->color.red) & 0xFF00) << 16) |
+	    (((int) (scale * canvas_link->color.green) & 0xFF00) << 8) |
+	    ((int) (scale * canvas_link->color.blue) & 0xFF00) | 0xFF;
+	  gnome_canvas_item_set (canvas_link->link_item, "points", points,
+				 "fill_color_rgba", scaledColor, NULL);
+	}
     }
+
   else
     {
-      /* scale color down to 10% at link timeout */
-      scale =
-	pow (0.10,
-	     (diff.tv_sec * 1000.0 +
-	      diff.tv_usec / 1000) / pref.link_timeout_time);
-      scaledColor =
-	(((int) (scale * canvas_link->color.red) & 0xFF00) << 16) |
-	(((int) (scale * canvas_link->color.green) & 0xFF00) << 8) |
-	((int) (scale * canvas_link->color.blue) & 0xFF00) | 0xFF;
+      guint32 black = 0x000000ff;
       gnome_canvas_item_set (canvas_link->link_item, "points", points,
-			     "fill_color_rgba", scaledColor, NULL);
+			     "fill_color_rgba", black, NULL);
     }
 
+
+
+
+
   /* If we got this far, the link can be shown. Make sure it is */
   gnome_canvas_item_show (canvas_link->link_item);
 
@@ -1355,7 +1367,8 @@ popup_to (struct popup_data *pd)
   GtkLabel *label;
   gchar *str;
 
-  xml_popup = glade_xml_new (GLADEDIR "/" ETHERAPE_GLADE_FILE, "node_popup");
+  xml_popup =
+    glade_xml_new (GLADEDIR "/" ETHERAPE_GLADE_FILE, "node_popup", NULL);
   glade_xml_signal_autoconnect (xml);
   pd->node_popup = glade_xml_get_widget (xml_popup, "node_popup");
 
