@@ -1,8 +1,8 @@
 /*
    Etherape
-   Copyright (C) 2000 Juan Toledo
    * 
-   *  File copied from mtr.
+   *  File copied from mtr. (www.bitwizard.nl/mtr) 
+   *  Copyright (C) 1997,1998  Matt Kimball
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,8 +24,5 @@
 void dns_open ();
 int dns_waitfd ();
 void dns_ack ();
-void dns_events (double *sinterval);
+unsigned int dns_tick ();	/* call this more or less every 10 secs */
 char *dns_lookup (int address, int fqdn);
-char *dns_lookup2 (int address);
-int dns_forward (char *name);
-char *strlongip (int address);
