@@ -1,6 +1,6 @@
 /* Etherape
  * Copyright (C) 2000 Juan Toledo
- * $Id: info_windows.h,v 1.5 2001/05/10 09:25:58 toledo Exp $
+ * $Id: info_windows.h,v 1.5.2.3 2003/02/25 22:09:50 bchiara Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,10 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <gtk/gtkbutton.h>
+#include <gtk/gtkclist.h>
+#include <gtk/gtkcheckmenuitem.h>
+
 typedef struct
 {
   guint8 *node_id;
@@ -31,14 +35,6 @@ typedef struct
 }
 prot_info_window_t;
 
-GList *node_info_windows = NULL;
-GList *prot_info_windows = NULL;
-
-static GList *info_protocols = NULL;
-
-static guint prot_clist_sort_column = 0;
-static gboolean prot_clist_reverse_sort = FALSE;
-
 
 static void update_prot_info_windows (void);
 
@@ -50,12 +46,8 @@ static void update_prot_info_window (prot_info_window_t * prot_info_window);
 static gint node_info_compare (gconstpointer a, gconstpointer b);
 static gint prot_info_compare (gconstpointer a, gconstpointer b);
 
-void on_node_info_delete_event (GtkWidget * node_info, gpointer user_data);
-void on_prot_info_delete_event (GtkWidget * node_info, gpointer user_data);
-static void prot_clist_button_clicked (GtkButton * button,
-				       gpointer func_data);
-static gint prot_window_compare (GtkCList * clist, gconstpointer p1,
-				 gconstpointer p2);
-static void create_prot_info_window (protocol_t * protocol);
-
-static gchar *timeval_to_str (struct timeval tv);
+gboolean on_node_info_delete_event (GtkWidget *, GdkEvent *, gpointer);
+gboolean on_prot_info_delete_event (GtkWidget *, GdkEvent *, gpointer);
+void toggle_protocols_window (void);
+gboolean on_prot_list_select_row (GtkTreeView * gv, gboolean arg1,
+				  gpointer user_data);
