@@ -1,12 +1,12 @@
 /* EtherApe
  * Copyright (C) 2001 Juan Toledo
- * $Id: menus.c,v 1.22 2001/08/02 09:49:08 toledo Exp $
+ * $Id: menus.c,v 1.22.2.4 2003/02/25 22:09:50 bchiara Exp $
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *
+ * 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -18,6 +18,10 @@
  */
 
 #include "menus.h"
+#include <string.h>
+#include <libgnomeui/gnome-file-entry.h>
+#include <gtk/gtkcontainer.h>
+#include "util.h"
 
 #if 0
 /* This is here just as an example in case I need it again */
@@ -27,7 +31,7 @@ on_file1_activate (GtkMenuItem * menuitem, gpointer user_data)
   GtkWidget *messagebox;
   GladeXML *xml_messagebox;
   xml_messagebox =
-    glade_xml_new (GLADEDIR "/" ETHERAPE_GLADE_FILE, "messagebox1");
+    glade_xml_new (GLADEDIR "/" ETHERAPE_GLADE_FILE, "messagebox1", NULL);
   messagebox = glade_xml_get_widget (xml_messagebox, "messagebox1");
   if (!xml)
     {
@@ -53,11 +57,11 @@ init_menus (void)
   /* It seems libglade is not acknowledging the "Use gnome help" option in the 
    * glade file and so it is not automatically adding the help items in the help
    * menu. Thus I must add it manually here */
-
+/* R.G.
   widget = glade_xml_get_widget (xml, "help1_menu");
   gnome_app_fill_menu ((GtkMenuShell *) widget, help_submenu,
 		       gtk_accel_group_get_default (), TRUE, 1);
-
+*/
   interface_list = get_interface_list (&err, err_str);
 
   interfaces = interface_list;
@@ -163,10 +167,11 @@ on_file_ok_button_clicked (GtkButton * button, gpointer user_data)
   if (str)
     g_free (str);
 
-  widget = glade_xml_get_widget (xml, "fileentry");
-  update_history (GNOME_ENTRY
-		  (gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY (widget))),
-		  pref.input_file, TRUE);
+  gnome_entry_append_history (GNOME_ENTRY
+			      (gnome_file_entry_gnome_entry
+			       (GNOME_FILE_ENTRY
+				(glade_xml_get_widget (xml, "fileentry")))),
+			      TRUE, pref.input_file);
 
   gui_start_capture ();
 
@@ -209,28 +214,30 @@ void
 on_mode_radio_activate (GtkMenuItem * menuitem, gpointer user_data)
 {
   apemode_t new_mode = DEFAULT;
+  const gchar *menuname = NULL;
 
-  g_assert (user_data != NULL);
 
   if (in_start_capture)
     return;			/* Disregard when called because
 				 * of interface look change from
 				 * start_capture */
 
+  menuname = gtk_widget_get_name (GTK_WIDGET (menuitem));
+  g_assert (menuname);
   g_my_debug ("Initial mode in on_mode_radio_activate %s",
-	      (gchar *) user_data);
+	      (gchar *) menuname);
 
-  if (!strcmp ("IEEE802", user_data))
+  if (!strcmp ("ieee802_radio", menuname))
     new_mode = IEEE802;
-  else if (!strcmp ("FDDI", user_data))
+  else if (!strcmp ("fddi_radio", menuname))
     new_mode = FDDI;
-  else if (!strcmp ("Ethernet", user_data))
+  else if (!strcmp ("ethernet_radio", menuname))
     new_mode = ETHERNET;
-  else if (!strcmp ("IP", user_data))
+  else if (!strcmp ("ip_radio", menuname))
     new_mode = IP;
-  else if (!strcmp ("TCP", user_data))
+  else if (!strcmp ("tcp_radio", menuname))
     new_mode = TCP;
-  else if (!strcmp ("UDP", user_data))
+  else if (!strcmp ("udp_radio", menuname))
     new_mode = UDP;
   else
     {
@@ -269,12 +276,13 @@ on_mode_radio_activate (GtkMenuItem * menuitem, gpointer user_data)
 	return;
       break;
     default:
+      return;
     }
   if (status != STOP)
     if (!gui_stop_capture ())
       return;
   pref.mode = new_mode;
-  g_my_info (_("Mode set to %s in GUI"), (gchar *) user_data);
+  g_my_info (_("Mode set to %s in GUI"), (gchar *) menuitem);
   gui_start_capture ();
 
 }				/* on_mode_radio_activate */
@@ -311,34 +319,25 @@ on_stop_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
 void
 on_toolbar_check_activate (GtkCheckMenuItem * menuitem, gpointer user_data)
 {
-  GnomeDock *dock;
   GtkWidget *widget;
 
-  dock = GNOME_DOCK (glade_xml_get_widget (xml, "dock1"));
   widget = glade_xml_get_widget (xml, "dock_toolbar");
   if (menuitem->active)
     gtk_widget_show (widget);
   else
     gtk_widget_hide (widget);
-
-  gtk_container_queue_resize (GTK_CONTAINER (dock));
 }				/* on_toolbar_check_activate */
 
 void
 on_legend_check_activate (GtkCheckMenuItem * menuitem, gpointer user_data)
 {
-  GnomeDock *dock;
   GtkWidget *widget;
 
-  dock = GNOME_DOCK (glade_xml_get_widget (xml, "dock1"));
   widget = glade_xml_get_widget (xml, "dock_legend");
   if (menuitem->active)
     gtk_widget_show (widget);
   else
     gtk_widget_hide (widget);
-
-  gtk_container_queue_resize (GTK_CONTAINER (dock));
-
 }				/* on_legend_check_activate */
 
 void
@@ -350,8 +349,6 @@ on_status_bar_check_activate (GtkCheckMenuItem * menuitem, gpointer user_data)
     gtk_widget_show (widget);
   else
     gtk_widget_hide (widget);
-
-  gtk_container_queue_resize (GTK_CONTAINER (app1));
 }				/* on_status_bar_check_activate */
 
 
@@ -366,7 +363,8 @@ on_about1_activate (GtkMenuItem * menuitem, gpointer user_data)
   GtkWidget *about;
   GladeXML *xml_about;
 
-  xml_about = glade_xml_new (GLADEDIR "/" ETHERAPE_GLADE_FILE, "about2");
+  xml_about =
+    glade_xml_new (GLADEDIR "/" ETHERAPE_GLADE_FILE, "about2", NULL);
   if (!xml_about)
     {
       g_error (_("We could not load the interface! (%s)"),
@@ -374,7 +372,7 @@ on_about1_activate (GtkMenuItem * menuitem, gpointer user_data)
       return;
     }
   about = glade_xml_get_widget (xml_about, "about2");
-  gtk_object_destroy (GTK_OBJECT (xml_about));
+  g_object_unref (xml_about);
 
   gtk_widget_show (about);
 
@@ -384,48 +382,6 @@ on_about1_activate (GtkMenuItem * menuitem, gpointer user_data)
 
 /* Helper functions */
 
-
-
-/* Saves the history of a gnome_entry making sure there are no duplicates */
-void
-update_history (GnomeEntry * gentry, const gchar * str, gboolean is_fileentry)
-{
-  GList *entry_items = NULL;
-  gboolean is_new = TRUE, normal_case = TRUE;
-
-  /* The combo fills up even if there are duplicate values. I think
-   * this is a bug in gnome and might be fixed in the future, but
-   * right now I'll just make sure the history is fine and reload that*/
-
-  /* TODO There really should be a better way of getting all values from
-   * the drop down list than this casting mess :-( */
-
-  /* The file entry introduces a copy of the file name at the end of the
-   * list if the browse button has been used. There goes another hack to
-   * work around it */
-  entry_items = GTK_LIST (GTK_COMBO (gentry)->list)->children;
-  while (entry_items)
-    {
-      gchar *history_value;
-      history_value = GTK_LABEL (GTK_BIN (entry_items->data)->child)->label;
-      if (!strcmp (history_value, str) && !is_fileentry
-	  && (entry_items->next != NULL))
-	is_new = FALSE;
-      normal_case = !(is_fileentry && !(entry_items->next));
-      entry_items = g_list_next (entry_items);
-    }
-
-  if (is_new)
-    {
-      g_my_info ("New entry history item: %s", str);
-      if (normal_case)
-	gnome_entry_prepend_history (gentry, TRUE, str);
-      gnome_entry_save_history (gentry);
-    }
-
-  gnome_entry_load_history (gentry);
-}				/* update_history */
-
 /* Sets up the GUI to reflect changes and calls start_capture() */
 void
 gui_start_capture (void)
@@ -508,6 +464,7 @@ gui_start_capture (void)
       gtk_widget_set_sensitive (widget, TRUE);
       break;
     default:
+      return;
     }
 
   /* Set active mode in GUI */
@@ -532,7 +489,6 @@ gui_start_capture (void)
     case UDP:
       widget = glade_xml_get_widget (xml, "udp_radio");
       break;
-    default:
     }
   gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (widget), TRUE);
 
@@ -570,7 +526,6 @@ gui_start_capture (void)
     case UDP:
       g_string_append (status_string, _(" in UDP mode"));
       break;
-    default:
     }
 
   set_appbar_status (status_string->str);
