@@ -1,6 +1,6 @@
 /* EtherApe
  * Copyright (C) 2001 Juan Toledo
- * $Id: menus.h,v 1.8 2001/07/06 10:14:00 toledo Exp $
+ * $Id: menus.h,v 1.8.2.1 2003/02/22 09:36:09 bchiara Exp $
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 #include <config.h>
 #endif
 
+#include <libgnomeui/gnome-popup-menu.h>
+#include <libgnomeui/gnome-messagebox.h>
 #include <gtk/gtk.h>
 #include "globals.h"
 #include "util.h"
