@@ -1,6 +1,6 @@
 /* EtherApe
  * Copyright (C) 2001 Juan Toledo
- * $Id: protocols.c,v 1.54 2001/08/08 17:00:22 toledo Exp $
+ * $Id: protocols.c,v 1.56 2002/10/04 18:39:18 bchiara Exp $
  *
  * This file is mostly a rehash of algorithms found in
  * packet-*. of ethereal
@@ -71,7 +71,7 @@ get_packet_prot (const guint8 * p, guint len)
       prot = g_string_append (prot, "NULL/IP");
       get_ip ();
       break;
-#ifdef L_LINUX_SLL
+#ifdef DLT_LINUX_SLL
     case L_LINUX_SLL:
       prot = g_string_append (prot, "LINUX-SLL");
       get_linux_sll_type ();
@@ -82,8 +82,8 @@ get_packet_prot (const guint8 * p, guint len)
     }
 
   /* TODO I think this is a serious waste of CPU and memory.
-     * I think I should look at other ways of dealing with not recognized
-     * protocols other than just creating entries saying UNKNOWN */
+   * I think I should look at other ways of dealing with not recognized
+   * protocols other than just creating entries saying UNKNOWN */
   /* I think I'll do this by changing the global protocols into
    * a GArray */
 
@@ -192,6 +192,7 @@ get_eth_802_3 (ethhdrtype_t ethhdr_type)
       get_ipx ();
       break;
     default:
+      break;
     }
 }				/* get_eth_802_3 */
 
