@@ -5,7 +5,7 @@
 /* util.c
  * Utility routines
  *
- * $Id: util.c,v 1.9 2001/04/18 01:03:56 toledo Exp $
+ * $Id: util.c,v 1.9.4.1 2003/02/22 09:36:09 bchiara Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -27,6 +27,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -92,9 +93,6 @@ typedef int mode_t;		/* for win32 */
 #include <sys/sockio.h>
 #endif
 
-#if 0				/* JTC */
-#include "globals.h"
-#endif /* JTC */
 
 #include <pcap.h> /*JTC*/
 #endif
